@@ -1,11 +1,12 @@
 /*-------------------------------------------------------------------------
  *
  * list.c
- *	  implementation for PostgreSQL generic linked list package
+ *	  implementation for PostgreSQL generic list package
  *
+ * See comments in pg_list.h
  *
- * Portions Copyright (c) 2003-2016, PgPool Global Development Group
- * Portions Copyright (c) 1996-2016, PostgreSQL Global Development Group
+ * Portions Copyright (c) 2003-2020, PgPool Global Development Group
+ * Portions Copyright (c) 1996-2020, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -15,13 +16,42 @@
  *-------------------------------------------------------------------------
  */
 
+#include <string.h>
 #include "utils/elog.h"
 #include <stdlib.h>
 #include "utils/palloc.h"
 #include "pg_list.h"
 
+static inline MemoryContext GetMemoryChunkContext(void *pointer);
+
 /*
- * Routines to simplify writing assertions about the type of a list; a
+ * The previous List implementation, since it used a separate palloc chunk
+ * for each cons cell, had the property that adding or deleting list cells
+ * did not move the storage of other existing cells in the list.  Quite a
+ * bit of existing code depended on that, by retaining ListCell pointers
+ * across such operations on a list.  There is no such guarantee in this
+ * implementation, so instead we have debugging support that is meant to
+ * help flush out now-broken assumptions.  Defining DEBUG_LIST_MEMORY_USAGE
+ * while building this file causes the List operations to forcibly move
+ * all cells in a list whenever a cell is added or deleted.  In combination
+ * with MEMORY_CONTEXT_CHECKING and/or Valgrind, this can usually expose
+ * broken code.  It's a bit expensive though, as there's many more palloc
+ * cycles and a lot more data-copying than in a default build.
+ *
+ * By default, we enable this when building for Valgrind.
+ */
+#ifdef USE_VALGRIND
+#define DEBUG_LIST_MEMORY_USAGE
+#endif
+
+#define VALGRIND_MAKE_MEM_NOACCESS(addr, size)              do {} while (0)
+
+/* Overhead for the fixed part of a List header, measured in ListCells */
+#define LIST_HEADER_OVERHEAD  \
+	((int) ((offsetof(List, initial_elements) - 1) / sizeof(ListCell) + 1))
+
+/*
+ * Macros to simplify writing assertions about the type of a list; a
  * NIL list is considered to be an empty list of any type.
  */
 #define IsPointerList(l)		((l) == NIL || IsA((l), List))
@@ -39,50 +69,180 @@ check_list_invariants(const List *list)
 		return;
 
 	Assert(list->length > 0);
-	Assert(list->head != NULL);
-	Assert(list->tail != NULL);
+	Assert(list->length <= list->max_length);
+	Assert(list->elements != NULL);
 
 	Assert(list->type == T_List ||
 		   list->type == T_IntList ||
 		   list->type == T_OidList);
-
-	if (list->length == 1)
-		Assert(list->head == list->tail);
-	if (list->length == 2)
-		Assert(list->head->next == list->tail);
-	Assert(list->tail->next == NULL);
 }
 #else
-#define check_list_invariants(l)
-#endif   /* USE_ASSERT_CHECKING */
+#define check_list_invariants(l)  ((void) 0)
+#endif							/* USE_ASSERT_CHECKING */
 
 /*
- * Return a freshly allocated List. Since empty non-NIL lists are
- * invalid, new_list() also allocates the head cell of the new list:
- * the caller should be sure to fill in that cell's data.
+ * Return a freshly allocated List with room for at least min_size cells.
+ *
+ * Since empty non-NIL lists are invalid, new_list() sets the initial length
+ * to min_size, effectively marking that number of cells as valid; the caller
+ * is responsible for filling in their data.
  */
 static List *
-new_list(NodeTag type)
+new_list(NodeTag type, int min_size)
 {
-	List	   *new_list;
-	ListCell   *new_head;
+	List	   *newlist;
+	int			max_size;
 
-	new_head = (ListCell *) palloc(sizeof(*new_head));
-	new_head->next = NULL;
-	/* new_head->data is left undefined! */
+	Assert(min_size > 0);
 
-	new_list = (List *) palloc(sizeof(*new_list));
-	new_list->type = type;
-	new_list->length = 1;
-	new_list->head = new_head;
-	new_list->tail = new_head;
+	/*
+	 * We allocate all the requested cells, and possibly some more, as part of
+	 * the same palloc request as the List header.  This is a big win for the
+	 * typical case of short fixed-length lists.  It can lose if we allocate a
+	 * moderately long list and then it gets extended; we'll be wasting more
+	 * initial_elements[] space than if we'd made the header small.  However,
+	 * rounding up the request as we do in the normal code path provides some
+	 * defense against small extensions.
+	 */
 
-	return new_list;
+	max_size = min_size;
+
+	newlist = (List *) palloc(offsetof(List, initial_elements) +
+							  max_size * sizeof(ListCell));
+	newlist->type = type;
+	newlist->length = min_size;
+	newlist->max_length = max_size;
+	newlist->elements = newlist->initial_elements;
+
+	return newlist;
 }
 
 /*
- * Allocate a new cell and make it the head of the specified
- * list. Assumes the list it is passed is non-NIL.
+ * Enlarge an existing non-NIL List to have room for at least min_size cells.
+ *
+ * This does *not* update list->length, as some callers would find that
+ * inconvenient.  (list->length had better be the correct number of existing
+ * valid cells, though.)
+ */
+static void
+enlarge_list(List *list, int min_size)
+{
+	int			new_max_len;
+
+	Assert(min_size > list->max_length);	/* else we shouldn't be here */
+
+	/* As above, don't allocate anything extra */
+	new_max_len = min_size;
+
+	if (list->elements == list->initial_elements)
+	{
+		/*
+		 * Replace original in-line allocation with a separate palloc block.
+		 * Ensure it is in the same memory context as the List header.  (The
+		 * previous List implementation did not offer any guarantees about
+		 * keeping all list cells in the same context, but it seems reasonable
+		 * to create such a guarantee now.)
+		 */
+		list->elements = (ListCell *)
+			MemoryContextAlloc(GetMemoryChunkContext(list),
+							   new_max_len * sizeof(ListCell));
+		memcpy(list->elements, list->initial_elements,
+			   list->length * sizeof(ListCell));
+
+		/*
+		 * We must not move the list header, so it's unsafe to try to reclaim
+		 * the initial_elements[] space via repalloc.  In debugging builds,
+		 * however, we can clear that space and/or mark it inaccessible.
+		 * (wipe_mem includes VALGRIND_MAKE_MEM_NOACCESS.)
+		 */
+#ifdef CLOBBER_FREED_MEMORY
+		wipe_mem(list->initial_elements,
+				 list->max_length * sizeof(ListCell));
+#else
+		VALGRIND_MAKE_MEM_NOACCESS(list->initial_elements,
+								   list->max_length * sizeof(ListCell));
+#endif
+	}
+	else
+	{
+#ifndef DEBUG_LIST_MEMORY_USAGE
+		/* Normally, let repalloc deal with enlargement */
+		list->elements = (ListCell *) repalloc(list->elements,
+											   new_max_len * sizeof(ListCell));
+#else
+		/*
+		 * repalloc() might enlarge the space in-place, which we don't want
+		 * for debugging purposes, so forcibly move the data somewhere else.
+		 */
+		ListCell   *newelements;
+
+		newelements = (ListCell *)
+			MemoryContextAlloc(GetMemoryChunkContext(list),
+							   new_max_len * sizeof(ListCell));
+		memcpy(newelements, list->elements,
+			   list->length * sizeof(ListCell));
+		pfree(list->elements);
+		list->elements = newelements;
+#endif
+	}
+
+	list->max_length = new_max_len;
+}
+
+/*
+ * Convenience functions to construct short Lists from given values.
+ * (These are normally invoked via the list_makeN macros.)
+ */
+List *
+list_make1_impl(NodeTag t, ListCell datum1)
+{
+	List	   *list = new_list(t, 1);
+
+	list->elements[0] = datum1;
+	check_list_invariants(list);
+	return list;
+}
+
+List *
+list_make2_impl(NodeTag t, ListCell datum1, ListCell datum2)
+{
+	List	   *list = new_list(t, 2);
+
+	list->elements[0] = datum1;
+	list->elements[1] = datum2;
+	check_list_invariants(list);
+	return list;
+}
+
+List *
+list_make3_impl(NodeTag t, ListCell datum1, ListCell datum2,
+				ListCell datum3)
+{
+	List	   *list = new_list(t, 3);
+
+	list->elements[0] = datum1;
+	list->elements[1] = datum2;
+	list->elements[2] = datum3;
+	check_list_invariants(list);
+	return list;
+}
+
+List *
+list_make4_impl(NodeTag t, ListCell datum1, ListCell datum2,
+				ListCell datum3, ListCell datum4)
+{
+	List	   *list = new_list(t, 4);
+
+	list->elements[0] = datum1;
+	list->elements[1] = datum2;
+	list->elements[2] = datum3;
+	list->elements[3] = datum4;
+	check_list_invariants(list);
+	return list;
+}
+
+/*
+ * Make room for a new head cell in the given (non-NIL) list.
  *
  * The data in the new head cell is undefined; the caller should be
  * sure to fill it in
@@ -90,18 +250,17 @@ new_list(NodeTag type)
 static void
 new_head_cell(List *list)
 {
-	ListCell   *new_head;
-
-	new_head = (ListCell *) palloc(sizeof(*new_head));
-	new_head->next = list->head;
-
-	list->head = new_head;
+	/* Enlarge array if necessary */
+	if (list->length >= list->max_length)
+		enlarge_list(list, list->length + 1);
+	/* Now shove the existing data over */
+	memmove(&list->elements[1], &list->elements[0],
+			list->length * sizeof(ListCell));
 	list->length++;
 }
 
 /*
- * Allocate a new cell and make it the tail of the specified
- * list. Assumes the list it is passed is non-NIL.
+ * Make room for a new tail cell in the given (non-NIL) list.
  *
  * The data in the new tail cell is undefined; the caller should be
  * sure to fill it in
@@ -109,13 +268,9 @@ new_head_cell(List *list)
 static void
 new_tail_cell(List *list)
 {
-	ListCell   *new_tail;
-
-	new_tail = (ListCell *) palloc(sizeof(*new_tail));
-	new_tail->next = NULL;
-
-	list->tail->next = new_tail;
-	list->tail = new_tail;
+	/* Enlarge array if necessary */
+	if (list->length >= list->max_length)
+		enlarge_list(list, list->length + 1);
 	list->length++;
 }
 
@@ -132,11 +287,11 @@ lappend(List *list, void *datum)
 	Assert(IsPointerList(list));
 
 	if (list == NIL)
-		list = new_list(T_List);
+		list = new_list(T_List, 1);
 	else
 		new_tail_cell(list);
 
-	lfirst(list->tail) = datum;
+	lfirst(list_tail(list)) = datum;
 	check_list_invariants(list);
 	return list;
 }
@@ -150,11 +305,11 @@ lappend_int(List *list, int datum)
 	Assert(IsIntegerList(list));
 
 	if (list == NIL)
-		list = new_list(T_IntList);
+		list = new_list(T_IntList, 1);
 	else
 		new_tail_cell(list);
 
-	lfirst_int(list->tail) = datum;
+	lfirst_int(list_tail(list)) = datum;
 	check_list_invariants(list);
 	return list;
 }
@@ -168,82 +323,83 @@ lappend_oid(List *list, Oid datum)
 	Assert(IsOidList(list));
 
 	if (list == NIL)
-		list = new_list(T_OidList);
+		list = new_list(T_OidList, 1);
 	else
 		new_tail_cell(list);
 
-	lfirst_oid(list->tail) = datum;
+	lfirst_oid(list_tail(list)) = datum;
 	check_list_invariants(list);
 	return list;
 }
 
 /*
- * Add a new cell to the list, in the position after 'prev_cell'. The
- * data in the cell is left undefined, and must be filled in by the
- * caller. 'list' is assumed to be non-NIL, and 'prev_cell' is assumed
- * to be non-NULL and a member of 'list'.
+ * Make room for a new cell at position 'pos' (measured from 0).
+ * The data in the cell is left undefined, and must be filled in by the
+ * caller. 'list' is assumed to be non-NIL, and 'pos' must be a valid
+ * list position, ie, 0 <= pos <= list's length.
+ * Returns address of the new cell.
  */
 static ListCell *
-add_new_cell(List *list, ListCell *prev_cell)
+insert_new_cell(List *list, int pos)
 {
-	ListCell   *new_cell;
-
-	new_cell = (ListCell *) palloc(sizeof(*new_cell));
-	/* new_cell->data is left undefined! */
-	new_cell->next = prev_cell->next;
-	prev_cell->next = new_cell;
-
-	if (list->tail == prev_cell)
-		list->tail = new_cell;
+	Assert(pos >= 0 && pos <= list->length);
 
+	/* Enlarge array if necessary */
+	if (list->length >= list->max_length)
+		enlarge_list(list, list->length + 1);
+	/* Now shove the existing data over */
+	if (pos < list->length)
+		memmove(&list->elements[pos + 1], &list->elements[pos],
+				(list->length - pos) * sizeof(ListCell));
 	list->length++;
 
-	return new_cell;
+	return &list->elements[pos];
 }
 
 /*
- * Add a new cell to the specified list (which must be non-NIL);
- * it will be placed after the list cell 'prev' (which must be
- * non-NULL and a member of 'list'). The data placed in the new cell
- * is 'datum'. The newly-constructed cell is returned.
+ * Insert the given datum at position 'pos' (measured from 0) in the list.
+ * 'pos' must be valid, ie, 0 <= pos <= list's length.
  */
-ListCell *
-lappend_cell(List *list, ListCell *prev, void *datum)
+List *
+list_insert_nth(List *list, int pos, void *datum)
 {
-	ListCell   *new_cell;
-
+	if (list == NIL)
+	{
+		Assert(pos == 0);
+		return list_make1(datum);
+	}
 	Assert(IsPointerList(list));
-
-	new_cell = add_new_cell(list, prev);
-	lfirst(new_cell) = datum;
+	lfirst(insert_new_cell(list, pos)) = datum;
 	check_list_invariants(list);
-	return new_cell;
+	return list;
 }
 
-ListCell *
-lappend_cell_int(List *list, ListCell *prev, int datum)
+List *
+list_insert_nth_int(List *list, int pos, int datum)
 {
-	ListCell   *new_cell;
-
+	if (list == NIL)
+	{
+		Assert(pos == 0);
+		return list_make1_int(datum);
+	}
 	Assert(IsIntegerList(list));
-
-	new_cell = add_new_cell(list, prev);
-	lfirst_int(new_cell) = datum;
+	lfirst_int(insert_new_cell(list, pos)) = datum;
 	check_list_invariants(list);
-	return new_cell;
+	return list;
 }
 
-ListCell *
-lappend_cell_oid(List *list, ListCell *prev, Oid datum)
+List *
+list_insert_nth_oid(List *list, int pos, Oid datum)
 {
-	ListCell   *new_cell;
-
+	if (list == NIL)
+	{
+		Assert(pos == 0);
+		return list_make1_oid(datum);
+	}
 	Assert(IsOidList(list));
-
-	new_cell = add_new_cell(list, prev);
-	lfirst_oid(new_cell) = datum;
+	lfirst_oid(insert_new_cell(list, pos)) = datum;
 	check_list_invariants(list);
-	return new_cell;
+	return list;
 }
 
 /*
@@ -263,11 +419,11 @@ lcons(void *datum, List *list)
 	Assert(IsPointerList(list));
 
 	if (list == NIL)
-		list = new_list(T_List);
+		list = new_list(T_List, 1);
 	else
 		new_head_cell(list);
 
-	lfirst(list->head) = datum;
+	lfirst(list_head(list)) = datum;
 	check_list_invariants(list);
 	return list;
 }
@@ -281,11 +437,11 @@ lcons_int(int datum, List *list)
 	Assert(IsIntegerList(list));
 
 	if (list == NIL)
-		list = new_list(T_IntList);
+		list = new_list(T_IntList, 1);
 	else
 		new_head_cell(list);
 
-	lfirst_int(list->head) = datum;
+	lfirst_int(list_head(list)) = datum;
 	check_list_invariants(list);
 	return list;
 }
@@ -299,46 +455,87 @@ lcons_oid(Oid datum, List *list)
 	Assert(IsOidList(list));
 
 	if (list == NIL)
-		list = new_list(T_OidList);
+		list = new_list(T_OidList, 1);
 	else
 		new_head_cell(list);
 
-	lfirst_oid(list->head) = datum;
+	lfirst_oid(list_head(list)) = datum;
 	check_list_invariants(list);
 	return list;
 }
 
 /*
- * Concatenate list2 to the end of list1, and return list1. list1 is
- * destructively changed. Callers should be sure to use the return
- * value as the new pointer to the concatenated list: the 'list1'
- * input pointer may or may not be the same as the returned pointer.
+ * Concatenate list2 to the end of list1, and return list1.
  *
- * The nodes in list2 are merely appended to the end of list1 in-place
- * (i.e. they aren't copied; the two lists will share some of the same
- * storage). Therefore, invoking list_free() on list2 will also
- * invalidate a portion of list1.
+ * This is equivalent to lappend'ing each element of list2, in order, to list1.
+ * list1 is destructively changed, list2 is not.  (However, in the case of
+ * pointer lists, list1 and list2 will point to the same structures.)
+ *
+ * Callers should be sure to use the return value as the new pointer to the
+ * concatenated list: the 'list1' input pointer may or may not be the same
+ * as the returned pointer.
  */
 List *
-list_concat(List *list1, List *list2)
+list_concat(List *list1, const List *list2)
 {
+	int			new_len;
+
 	if (list1 == NIL)
-		return list2;
+		return list_copy(list2);
 	if (list2 == NIL)
 		return list1;
-	if (list1 == list2)
-		elog(ERROR, "cannot list_concat() a list to itself");
 
 	Assert(list1->type == list2->type);
 
-	list1->length += list2->length;
-	list1->tail->next = list2->head;
-	list1->tail = list2->tail;
+	new_len = list1->length + list2->length;
+	/* Enlarge array if necessary */
+	if (new_len > list1->max_length)
+		enlarge_list(list1, new_len);
+
+	/* Even if list1 == list2, using memcpy should be safe here */
+	memcpy(&list1->elements[list1->length], &list2->elements[0],
+		   list2->length * sizeof(ListCell));
+	list1->length = new_len;
 
 	check_list_invariants(list1);
 	return list1;
 }
 
+/*
+ * Form a new list by concatenating the elements of list1 and list2.
+ *
+ * Neither input list is modified.  (However, if they are pointer lists,
+ * the output list will point to the same structures.)
+ *
+ * This is equivalent to, but more efficient than,
+ * list_concat(list_copy(list1), list2).
+ * Note that some pre-v13 code might list_copy list2 as well, but that's
+ * pointless now.
+ */
+List *
+list_concat_copy(const List *list1, const List *list2)
+{
+	List	   *result;
+	int			new_len;
+
+	if (list1 == NIL)
+		return list_copy(list2);
+	if (list2 == NIL)
+		return list_copy(list1);
+
+	Assert(list1->type == list2->type);
+
+	new_len = list1->length + list2->length;
+	result = new_list(list1->type, new_len);
+	memcpy(result->elements, list1->elements,
+		   list1->length * sizeof(ListCell));
+	memcpy(result->elements + list1->length, list2->elements,
+		   list2->length * sizeof(ListCell));
+
+	check_list_invariants(result);
+	return result;
+}
+
 /*
  * Truncate 'list' to contain no more than 'new_size' elements. This
  * modifies the list in-place! Despite this, callers should use the
@@ -351,91 +548,26 @@ list_concat(List *list1, List *list2)
 List *
 list_truncate(List *list, int new_size)
 {
-	ListCell   *cell;
-	int			n;
-
 	if (new_size <= 0)
 		return NIL;				/* truncate to zero length */
 
 	/* If asked to effectively extend the list, do nothing */
-	if (new_size >= list_length(list))
-		return list;
+	if (new_size < list_length(list))
+		list->length = new_size;
 
-	n = 1;
-	foreach(cell, list)
-	{
-		if (n == new_size)
-		{
-			cell->next = NULL;
-			list->tail = cell;
-			list->length = new_size;
-			check_list_invariants(list);
-			return list;
-		}
-		n++;
-	}
+	/*
+	 * Note: unlike the individual-list-cell deletion functions, we don't move
+	 * the list cells to new storage, even in DEBUG_LIST_MEMORY_USAGE mode.
+	 * This is because none of them can move in this operation, so just like
+	 * in the old cons-cell-based implementation, this function doesn't
+	 * invalidate any pointers to cells of the list.  This is also the reason
+	 * for not wiping the memory of the deleted cells: the old code didn't
+	 * free them either.  Perhaps later we'll tighten this up.
+	 */
 
-	/* keep the compiler quiet; never reached */
-	Assert(false);
 	return list;
 }
 
-/*
- * Locate the n'th cell (counting from 0) of the list.  It is an assertion
- * failure if there is no such cell.
- */
-ListCell *
-list_nth_cell(const List *list, int n)
-{
-	ListCell   *match;
-
-	Assert(list != NIL);
-	Assert(n >= 0);
-	Assert(n < list->length);
-	check_list_invariants(list);
-
-	/* Does the caller actually mean to fetch the tail? */
-	if (n == list->length - 1)
-		return list->tail;
-
-	for (match = list->head; n-- > 0; match = match->next)
-		;
-
-	return match;
-}
-
-/*
- * Return the data value contained in the n'th element of the
- * specified list. (List elements begin at 0.)
- */
-void *
-list_nth(const List *list, int n)
-{
-	Assert(IsPointerList(list));
-	return lfirst(list_nth_cell(list, n));
-}
-
-/*
- * Return the integer value contained in the n'th element of the
- * specified list.
- */
-int
-list_nth_int(const List *list, int n)
-{
-	Assert(IsIntegerList(list));
-	return lfirst_int(list_nth_cell(list, n));
-}
-
-/*
- * Return the OID value contained in the n'th element of the specified
- * list.
- */
-Oid
-list_nth_oid(const List *list, int n)
-{
-	Assert(IsOidList(list));
-	return lfirst_oid(list_nth_cell(list, n));
-}
 #ifdef NOT_USED_IN_PGPOOL
 /*
  * Return true iff 'datum' is a member of the list. Equality is
@@ -458,7 +590,7 @@ list_member(const List *list, const void *datum)
 
 	return false;
 }
-#endif /*NOT_USED_IN_PGPOOL*/
+#endif							/* NOT_USED_IN_PGPOOL */
 /*
  * Return true iff 'datum' is a member of the list. Equality is
  * determined by using simple pointer comparison.
@@ -521,16 +653,16 @@ list_member_oid(const List *list, Oid datum)
 }
 
 /*
- * Delete 'cell' from 'list'; 'prev' is the previous element to 'cell'
- * in 'list', if any (i.e. prev == NULL iff list->head == cell)
+ * Delete the n'th cell (counting from 0) in list.
  *
- * The cell is pfree'd, as is the List header if this was the last member.
+ * The List is pfree'd if this was the last member.
  */
 List *
-list_delete_cell(List *list, ListCell *cell, ListCell *prev)
+list_delete_nth_cell(List *list, int n)
 {
 	check_list_invariants(list);
-	Assert(prev != NULL ? lnext(prev) == cell : list_head(list) == cell);
+
+	Assert(n >= 0 && n < list->length);
 
 	/*
 	 * If we're about to delete the last node from the list, free the whole
@@ -544,23 +676,64 @@ list_delete_cell(List *list, ListCell *cell, ListCell *prev)
 	}
 
 	/*
-	 * Otherwise, adjust the necessary list links, deallocate the particular
-	 * node we have just removed, and return the list we were given.
+	 * Otherwise, we normally just collapse out the removed element.  But for
+	 * debugging purposes, move the whole list contents someplace else.
+	 *
+	 * (Note that we *must* keep the contents in the same memory context.)
 	 */
+#ifndef DEBUG_LIST_MEMORY_USAGE
+	memmove(&list->elements[n], &list->elements[n + 1],
+			(list->length - 1 - n) * sizeof(ListCell));
 	list->length--;
+#else
+	{
+		ListCell   *newelems;
+		int			newmaxlen = list->length - 1;
 
-	if (prev)
-		prev->next = cell->next;
-	else
-		list->head = cell->next;
+		newelems = (ListCell *)
+			MemoryContextAlloc(GetMemoryChunkContext(list),
+							   newmaxlen * sizeof(ListCell));
+		memcpy(newelems, list->elements, n * sizeof(ListCell));
+		memcpy(&newelems[n], &list->elements[n + 1],
+			   (list->length - 1 - n) * sizeof(ListCell));
+		if (list->elements != list->initial_elements)
+			pfree(list->elements);
+		else
+		{
+			/*
+			 * As in enlarge_list(), clear the initial_elements[] space and/or
+			 * mark it inaccessible.
+			 */
+#ifdef CLOBBER_FREED_MEMORY
+			wipe_mem(list->initial_elements,
+					 list->max_length * sizeof(ListCell));
+#else
+			VALGRIND_MAKE_MEM_NOACCESS(list->initial_elements,
+									   list->max_length * sizeof(ListCell));
+#endif
+		}
+		list->elements = newelems;
+		list->max_length = newmaxlen;
+		list->length--;
+		check_list_invariants(list);
+	}
+#endif
 
-	if (list->tail == cell)
-		list->tail = prev;
-
-	pfree(cell);
 	return list;
 }
 
+/*
+ * Delete 'cell' from 'list'.
+ *
+ * The List is pfree'd if this was the last member.  However, we do not
+ * touch any data the cell might've been pointing to.
+ */
+List *
+list_delete_cell(List *list, ListCell *cell)
+{
+	return list_delete_nth_cell(list, cell - list->elements);
+}
+
 #ifdef NOT_USED_IN_PGPOOL
 
 /*
@@ -571,43 +744,35 @@ List *
 list_delete(List *list, void *datum)
 {
 	ListCell   *cell;
-	ListCell   *prev;
 
 	Assert(IsPointerList(list));
 	check_list_invariants(list);
 
-	prev = NULL;
 	foreach(cell, list)
 	{
 		if (equal(lfirst(cell), datum))
-			return list_delete_cell(list, cell, prev);
-
-		prev = cell;
+			return list_delete_cell(list, cell);
 	}
 
 	/* Didn't find a match: return the list unmodified */
 	return list;
 }
 
-#endif /*NOT_USED_IN_PGPOOL*/
+#endif							/* NOT_USED_IN_PGPOOL */
 
 /* As above, but use simple pointer equality */
 List *
 list_delete_ptr(List *list, void *datum)
 {
 	ListCell   *cell;
-	ListCell   *prev;
 
 	Assert(IsPointerList(list));
 	check_list_invariants(list);
 
-	prev = NULL;
 	foreach(cell, list)
 	{
 		if (lfirst(cell) == datum)
-			return list_delete_cell(list, cell, prev);
-
-		prev = cell;
+			return list_delete_cell(list, cell);
 	}
 
 	/* Didn't find a match: return the list unmodified */
@@ -619,18 +784,14 @@ List *
 list_delete_int(List *list, int datum)
 {
 	ListCell   *cell;
-	ListCell   *prev;
 
 	Assert(IsIntegerList(list));
 	check_list_invariants(list);
 
-	prev = NULL;
 	foreach(cell, list)
 	{
 		if (lfirst_int(cell) == datum)
-			return list_delete_cell(list, cell, prev);
-
-		prev = cell;
+			return list_delete_cell(list, cell);
 	}
 
 	/* Didn't find a match: return the list unmodified */
@@ -642,18 +803,14 @@ List *
 list_delete_oid(List *list, Oid datum)
 {
 	ListCell   *cell;
-	ListCell   *prev;
 
 	Assert(IsOidList(list));
 	check_list_invariants(list);
 
-	prev = NULL;
 	foreach(cell, list)
 	{
 		if (lfirst_oid(cell) == datum)
-			return list_delete_cell(list, cell, prev);
-
-		prev = cell;
+			return list_delete_cell(list, cell);
 	}
 
 	/* Didn't find a match: return the list unmodified */
@@ -665,8 +822,8 @@ list_delete_oid(List *list, Oid datum)
  *
  * This is useful to replace the Lisp-y code "list = lnext(list);" in cases
  * where the intent is to alter the list rather than just traverse it.
- * Beware that the removed cell is freed, whereas the lnext() coding leaves
- * the original list head intact if there's another pointer to it.
+ * Beware that the list is modified, whereas the Lisp-y coding leaves
+ * the original list head intact in case there's another pointer to it.
  */
 List *
 list_delete_first(List *list)
@@ -676,7 +833,31 @@ list_delete_first(List *list)
 	if (list == NIL)
 		return NIL;				/* would an error be better? */
 
-	return list_delete_cell(list, list_head(list), NULL);
+	return list_delete_nth_cell(list, 0);
+}
+
+/*
+ * Delete the last element of the list.
+ *
+ * This is the opposite of list_delete_first(), but is noticeably cheaper
+ * with a long list, since no data need be moved.
+ */
+List *
+list_delete_last(List *list)
+{
+	check_list_invariants(list);
+
+	if (list == NIL)
+		return NIL;				/* would an error be better? */
+
+	/* list_truncate won't free list if it goes to empty, but this should */
+	if (list_length(list) <= 1)
+	{
+		list_free(list);
+		return NIL;
+	}
+
+	return list_truncate(list, list_length(list) - 1);
 }
 
 #ifdef NOT_USED_IN_PGPOOL
@@ -695,7 +876,7 @@ list_delete_first(List *list)
  * in list1 (so it only performs a "union" if list1 is known unique to
  * start with).  Also, if you are about to write "x = list_union(x, y)"
  * you probably want to use list_concat_unique() instead to avoid wasting
- * the list cells of the old x list.
+ * the storage of the old x list.
  *
  * This function could probably be implemented a lot faster if it is a
  * performance bottleneck.
@@ -719,7 +900,7 @@ list_union(const List *list1, const List *list2)
 	check_list_invariants(result);
 	return result;
 }
-#endif /*NOT_USED_IN_PGPOOL*/
+#endif							/* NOT_USED_IN_PGPOOL */
 
 /*
  * This variant of list_union() determines duplicates via simple
@@ -884,7 +1065,7 @@ list_difference(const List *list1, const List *list2)
 	check_list_invariants(result);
 	return result;
 }
-#endif /*NOT_USED_IN_PGPOOL*/
+#endif							/* NOT_USED_IN_PGPOOL */
 
 /*
  * This variant of list_difference() determines list membership via
@@ -979,7 +1160,7 @@ list_append_unique(List *list, void *datum)
 		return lappend(list, datum);
 }
 
-#endif /*NOT_USED_IN_PGPOOL*/
+#endif							/* NOT_USED_IN_PGPOOL */
 
 /*
  * This variant of list_append_unique() determines list membership via
@@ -1026,11 +1207,12 @@ list_append_unique_oid(List *list, Oid datum)
  * via equal().
  *
  * This is almost the same functionality as list_union(), but list1 is
- * modified in-place rather than being copied.  Note also that list2's cells
- * are not inserted in list1, so the analogy to list_concat() isn't perfect.
+ * modified in-place rather than being copied. However, callers of this
+ * function may have strict ordering expectations -- i.e. that the relative
+ * order of those list2 elements that are not duplicates is preserved.
  */
 List *
-list_concat_unique(List *list1, List *list2)
+list_concat_unique(List *list1, const List *list2)
 {
 	ListCell   *cell;
 
@@ -1046,14 +1228,14 @@ list_concat_unique(List *list1, List *list2)
 	check_list_invariants(list1);
 	return list1;
 }
-#endif /*NOT_USED_IN_PGPOOL*/
+#endif							/* NOT_USED_IN_PGPOOL */
 
 /*
  * This variant of list_concat_unique() determines list membership via
  * simple pointer equality.
  */
 List *
-list_concat_unique_ptr(List *list1, List *list2)
+list_concat_unique_ptr(List *list1, const List *list2)
 {
 	ListCell   *cell;
 
@@ -1074,7 +1256,7 @@ list_concat_unique_ptr(List *list1, List *list2)
  * This variant of list_concat_unique() operates upon lists of integers.
  */
 List *
-list_concat_unique_int(List *list1, List *list2)
+list_concat_unique_int(List *list1, const List *list2)
 {
 	ListCell   *cell;
 
@@ -1095,7 +1277,7 @@ list_concat_unique_int(List *list1, List *list2)
  * This variant of list_concat_unique() operates upon lists of OIDs.
  */
 List *
-list_concat_unique_oid(List *list1, List *list2)
+list_concat_unique_oid(List *list1, const List *list2)
 {
 	ListCell   *cell;
 
@@ -1112,29 +1294,53 @@ list_concat_unique_oid(List *list1, List *list2)
 	return list1;
 }
 
+/*
+ * Remove adjacent duplicates in a list of OIDs.
+ *
+ * It is caller's responsibility to have sorted the list to bring duplicates
+ * together, perhaps via list_sort(list, list_oid_cmp).
+ */
+void
+list_deduplicate_oid(List *list)
+{
+	int			len;
+
+	Assert(IsOidList(list));
+	len = list_length(list);
+	if (len > 1)
+	{
+		ListCell   *elements = list->elements;
+		int			i = 0;
+
+		for (int j = 1; j < len; j++)
+		{
+			if (elements[i].oid_value != elements[j].oid_value)
+				elements[++i].oid_value = elements[j].oid_value;
+		}
+		list->length = i + 1;
+	}
+	check_list_invariants(list);
+}
+
 /*
  * Free all storage in a list, and optionally the pointed-to elements
  */
 static void
 list_free_private(List *list, bool deep)
 {
-	ListCell   *cell;
+	if (list == NIL)
+		return;					/* nothing to do */
 
 	check_list_invariants(list);
 
-	cell = list_head(list);
-	while (cell != NULL)
+	if (deep)
 	{
-		ListCell   *tmp = cell;
-
-		cell = lnext(cell);
-		if (deep)
-			pfree(lfirst(tmp));
-		pfree(tmp);
+		for (int i = 0; i < list->length; i++)
+			pfree(lfirst(&list->elements[i]));
 	}
-
-	if (list)
-		pfree(list);
+	if (list->elements != list->initial_elements)
+		pfree(list->elements);
+	pfree(list);
 }
 
 /*
@@ -1176,37 +1382,13 @@ List *
 list_copy(const List *oldlist)
 {
 	List	   *newlist;
-	ListCell   *newlist_prev;
-	ListCell   *oldlist_cur;
 
 	if (oldlist == NIL)
 		return NIL;
 
-	newlist = new_list(oldlist->type);
-	newlist->length = oldlist->length;
-
-	/*
-	 * Copy over the data in the first cell; new_list() has already allocated
-	 * the head cell itself
-	 */
-	newlist->head->data = oldlist->head->data;
-
-	newlist_prev = newlist->head;
-	oldlist_cur = oldlist->head->next;
-	while (oldlist_cur)
-	{
-		ListCell   *newlist_cur;
-
-		newlist_cur = (ListCell *) palloc(sizeof(*newlist_cur));
-		newlist_cur->data = oldlist_cur->data;
-		newlist_prev->next = newlist_cur;
-
-		newlist_prev = newlist_cur;
-		oldlist_cur = oldlist_cur->next;
-	}
-
-	newlist_prev->next = NULL;
-	newlist->tail = newlist_prev;
+	newlist = new_list(oldlist->type, oldlist->length);
+	memcpy(newlist->elements, oldlist->elements,
+		   newlist->length * sizeof(ListCell));
 
 	check_list_invariants(newlist);
 	return newlist;
@@ -1219,8 +1401,6 @@ List *
 list_copy_tail(const List *oldlist, int nskip)
 {
 	List	   *newlist;
-	ListCell   *newlist_prev;
-	ListCell   *oldlist_cur;
 
 	if (nskip < 0)
 		nskip = 0;				/* would it be better to elog? */
@@ -1228,63 +1408,116 @@ list_copy_tail(const List *oldlist, int nskip)
 	if (oldlist == NIL || nskip >= oldlist->length)
 		return NIL;
 
-	newlist = new_list(oldlist->type);
-	newlist->length = oldlist->length - nskip;
-
-	/*
-	 * Skip over the unwanted elements.
-	 */
-	oldlist_cur = oldlist->head;
-	while (nskip-- > 0)
-		oldlist_cur = oldlist_cur->next;
-
-	/*
-	 * Copy over the data in the first remaining cell; new_list() has already
-	 * allocated the head cell itself
-	 */
-	newlist->head->data = oldlist_cur->data;
-
-	newlist_prev = newlist->head;
-	oldlist_cur = oldlist_cur->next;
-	while (oldlist_cur)
-	{
-		ListCell   *newlist_cur;
-
-		newlist_cur = (ListCell *) palloc(sizeof(*newlist_cur));
-		newlist_cur->data = oldlist_cur->data;
-		newlist_prev->next = newlist_cur;
-
-		newlist_prev = newlist_cur;
-		oldlist_cur = oldlist_cur->next;
-	}
-
-	newlist_prev->next = NULL;
-	newlist->tail = newlist_prev;
+	newlist = new_list(oldlist->type, oldlist->length - nskip);
+	memcpy(newlist->elements, &oldlist->elements[nskip],
+		   newlist->length * sizeof(ListCell));
 
 	check_list_invariants(newlist);
 	return newlist;
 }
 
 /*
- * Temporary compatibility functions
+ * Return a deep copy of the specified list.
  *
- * In order to avoid warnings for these function definitions, we need
- * to include a prototype here as well as in pg_list.h. That's because
- * we don't enable list API compatibility in list.c, so we
- * don't see the prototypes for these functions.
+ * The list elements are copied via copyObject(), so that this function's
+ * idea of a "deep" copy is considerably deeper than what list_free_deep()
+ * means by the same word.
  */
+List *
+list_copy_deep(const List *oldlist)
+{
+	List	   *newlist;
+
+	if (oldlist == NIL)
+		return NIL;
+
+	/* This is only sensible for pointer Lists */
+	Assert(IsA(oldlist, List));
+
+	newlist = new_list(oldlist->type, oldlist->length);
+	for (int i = 0; i < newlist->length; i++)
+		lfirst(&newlist->elements[i]) =
+			copyObjectImpl(lfirst(&oldlist->elements[i]));
+
+	check_list_invariants(newlist);
+	return newlist;
+}
 
 /*
- * Given a list, return its length. This is merely defined for the
- * sake of backward compatibility: we can't afford to define a macro
- * called "length", so it must be a function. New code should use the
- * list_length() macro in order to avoid the overhead of a function
- * call.
+ * Sort a list according to the specified comparator function.
+ *
+ * The list is sorted in-place.
+ *
+ * The comparator function is declared to receive arguments of type
+ * const ListCell *; this allows it to use lfirst() and variants
+ * without casting its arguments.  Otherwise it behaves the same as
+ * the comparator function for standard qsort().
+ *
+ * Like qsort(), this provides no guarantees about sort stability
+ * for equal keys.
  */
-int			length(const List *list);
-
-int
-length(const List *list)
+void
+list_sort(List *list, list_sort_comparator cmp)
 {
-	return list_length(list);
+	typedef int (*qsort_comparator) (const void *a, const void *b);
+	int			len;
+
+	check_list_invariants(list);
+
+	/* Nothing to do if there's less than two elements */
+	len = list_length(list);
+	if (len > 1)
+		qsort(list->elements, len, sizeof(ListCell), (qsort_comparator) cmp);
 }
+
+/*
+ * list_sort comparator for sorting a list into ascending OID order.
+ */
+int
+list_oid_cmp(const ListCell *p1, const ListCell *p2)
+{
+	Oid			v1 = lfirst_oid(p1);
+	Oid			v2 = lfirst_oid(p2);
+
+	if (v1 < v2)
+		return -1;
+	if (v1 > v2)
+		return 1;
+	return 0;
+}
+
+/*
+ * GetMemoryChunkContext
+ *      Given a currently-allocated chunk, determine the context
+ *      it belongs to.
+ *
+ * All chunks allocated by any memory context manager are required to be
+ * preceded by the corresponding MemoryContext stored, without padding, in the
+ * preceding sizeof(void*) bytes.  A currently-allocated chunk must contain a
+ * backpointer to its owning context.  The backpointer is used by pfree() and
+ * repalloc() to find the context to call.
+ */
+#ifndef FRONTEND
+static inline MemoryContext
+GetMemoryChunkContext(void *pointer)
+{
+    MemoryContext context;
+
+    /*
+     * Try to detect bogus pointers handed to us, poorly though we can.
+     * Presumably, a pointer that isn't MAXALIGNED isn't pointing at an
+     * allocated chunk.
+     */
+    Assert(pointer != NULL);
+    Assert(pointer == (void *) MAXALIGN(pointer));
+
+    /*
+     * OK, it's probably safe to look at the context.
+     */
+    context = *(MemoryContext *) (((char *) pointer) - sizeof(void *));
+
+    AssertArg(MemoryContextIsValid(context));
+
+    return context;
+}
+#endif
