@@ -1017,11 +1017,11 @@ wd_ping_pgpool(LifeCheckNode * node, char* password)
 	conn = create_conn(node->hostName, node->pgpoolPort, password);
 	if (conn == NULL)
 	{
-		if(chceck_password_type_is_not_md5(pool_config->wd_lifecheck_user, pool_config->wd_lifecheck_password) == -1)
+		if(check_password_type_is_not_md5(pool_config->wd_lifecheck_user, pool_config->wd_lifecheck_password) == -1)
 		{
 			ereport(ERROR,
 					(errmsg("the password of wd_lifecheck_user %s is invalid format",
-						pool_config->recovery_user),
+						pool_config->wd_lifecheck_user),
 					errdetail("wd_lifecheck_password is not allowed to be md5 hashed format")));
 		}
 		return WD_NG;
